/**
 * This class provides access to device gestures.
 * This class is for Bada use.
 * Currently supports only shake gesture.
 * @constructor
 */
var Gestures = function(){
	/**
	 * Asynchronously registers to the shake event.
	 *
	 * @param {Function} successCallback    The function to call when the extended device data is available
	 * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
	 * @param {Function} cbEventHandler      The function to call when there is an error getting the acceleration data. (OPTIONAL)
	 */
	this.prototype.registerShake = function(successCallback, errorCallback, cbEventHandler){
	
		// successCallback required
		if (typeof successCallback != "function") {
			console.log("Gestures Error: successCallback is not a function");
			return;
		}
		
		// errorCallback required
		if (errorCallback && (typeof errorCallback != "function")) {
			console.log("Gestures Error: errorCallback is not a function");
			return;
		}
		
		this.cbShakeEvent = cbEventHandler;
		
		// Get acceleration
		PhoneGap.exec(successCallback, errorCallback, "Gestures", "registerShake", []);
	};
	
	/**
	 * Asynchronously registers to the shake event.
	 *
	 * @param {Function} successCallback    The function to call when the extended device data is available
	 * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
	 * @param {Function} cbEventHandler      The function to call when there is an error getting the acceleration data. (OPTIONAL)
	 */
	this.prototype.unregisterShake = function(successCallback, errorCallback){
	
		// successCallback required
		if (typeof successCallback != "function") {
			console.log("Gestures Error: successCallback is not a function");
			return;
		}
		
		// errorCallback required
		if (errorCallback && (typeof errorCallback != "function")) {
			console.log("Gestures Error: errorCallback is not a function");
			return;
		}
		
		// Get acceleration
		PhoneGap.exec(successCallback, errorCallback, "Gestures", "unregisterShake", []);
	};
	
	this.prototype.shakeEvent = function(){
		this.cbShakeEvent();
	};
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.gestures == "undefined") 
	{
		navigator.gestures = new Gestures();
	}
});

