/**
 * 
 */
// Initialize puzzle application
function initApp() 
{
	conduitApi.init(
	function () 
	{
		_app.init();
	
		_image.init();
		_options.init(function () 
		{
			_game.init();
		});
	});
}

var _app = {
	// Switch between display of menus
	switchDisplay: function (fromId, toId) {
		var from = $(fromId);
		var to = $(toId);

		from.style.display = "none";
		to.style.display = "block";
	},
	
	init: function() {
		this._setupInfoBar();
		
		this._setupMainMenuButtons();
		this._setupCancelButtons();
	},
	
	// Setup the info bar buttons
	_setupInfoBar: function() {
		// Hide Info Bar
		conduitApi.app.hideInfoBar(empty, empty);
		
		// Add main menu button
		conduitApi.app
			.addInfoBarButton(
				empty, empty, 
				function() {
					_game.hideGame('mainMenu');
				},
				'images/main-menu.png',	false);
		
		// Add options button
		conduitApi.app
			.addInfoBarButton(
				empty, empty, 
				function() {
					_game.hideGame('optionsMenu');
				},
				'images/options.png', true);		
	},
	
	// Setup click events for main menu buttons
	_setupMainMenuButtons: function() {
		var newGameButton = $('newGameButton');
		newGameButton.onclick = function() {
			_game.newGame();
		}
		
		var optionsMenuButton = $('optionsButton');
		optionsMenuButton.onclick = function() {
			_options._setOptions();
			_app.switchDisplay('mainMenu', 'optionsMenu');
		};
		
		var selectImageMenuButton = $('selectImageButton');
		selectImageMenuButton.onclick = function() {
			_app.switchDisplay('mainMenu', 'selectImageMenu');
		};
	},
	
	// Setup click events for close and cancel buttons
	_setupCancelButtons: function() {
		var optionsMenuCancel = $('cancelOptionsButton');
		var optionsMenuClose = $('closeOptionsButton');
		var closeOptions = function() {
			_app.switchDisplay('optionsMenu', 'mainMenu');
		};
		optionsMenuCancel.onclick = closeOptions;
		optionsMenuClose.onclick = closeOptions;
		
		var selectImageMenuCancel = $('cancelSelectImageButton');
		var selectImageMenuClose = $('closeSelectImageButton');
		var closeSelectImage = function() {
			_app.switchDisplay( 'selectImageMenu', 'mainMenu');
		};
		selectImageMenuCancel.onclick = closeSelectImage;
		selectImageMenuClose.onclick = closeSelectImage;		
	}
};

var _image = {
	uri: "",
	previewUri: "",
	defaultUri: "images/puzzle01.jpg",
	defaultPreviewUri: "images/no-preview.jpg",
	defaultWidth: "278px",
	defaultHeight: "350px",

	init: function() {
		var captureButton = $('captureImageButton');
		captureButton.onclick = function() {
			var source = $('imageSource');
			_image.selectImage(source.selectedIndex);
		};

		var saveButton = $('saveSelectImageButton');
		saveButton.onclick = function() {
			_image.setPuzzleImage();
		}
	},

	selectImage: function(source) {		
		var success = function( imageData ) {		
			_image.uri = imageData;
			_image.previewUri = imageData;
			
			_image.setPreviewImage();
		};

		var fail = function() {
			_image.uri = _image.defaultUri;
			_image.previewUri = _image.defaultPreviewUri;
			
			_image.setPreviewImage();
		};

		var cameraOptions = {
			destinationType : 1,
			sourceType : source,
			allowEdit : true
		};

		conduitApi.platform.camera
			.captureImage(success, fail, cameraOptions);
	},

	setPreviewImage: function() {
		$('previewImage').src = this.previewUri;
	},

	setPuzzleImage: function() {
		puzzle.changeImage( this.uri );
		
		_app.switchDisplay( 'selectImageMenu', 'mainMenu');
	}
};

var _options = {
	level: 1,
	orientationLock: false,
	degree: 0,
	accelerometerSupport: false,
	
	_isDirty: true,
	
	init: function(cbInitDone) {		
		this._readOptions(function () 
		{
			_options._applyOptions( false );
			
			var saveButton = $( 'saveOptionsButton' );
			saveButton.onclick = function() {
			    _options._saveOptions();
			};
			cbInitDone();
		},
		empty);
	},
	
	_saveOptions: function() {    
	    this._getOptions();
	    this._applyOptions( true );
	    this._writeOptions();
	    
	    _app.switchDisplay( 'optionsMenu', 'mainMenu' );
	},
	
	_getOptions: function() {
		var levelSelector = $( 'level' );
		if( levelSelector.selectedIndex != this.level ){
			this.level = levelSelector.selectedIndex;
			this._isDirty = true;
		}
		
		var orientation = $( 'orientLock' );
		if( orientation.checked != this.orientationLock ) {
			this.orientationLock = orientation.checked;
			if( this.orientatinLock ){
				conduitApi.platform.orientation
					.getOrientation( 
						function( curOrientation ){
							this.degree = curOrientation;
							this._isDirty = true;
						},
						empty );				
			}
		}
		
		var accelerometer = $( 'accelSupport' );
		if( accelerometer.checked != this.accelerometerSupport ) {
			this.accelerometerSupport = accelerometer.checked;
			this._isDirty = true;
		}
	},
	
	_setOptions: function() {
		var levelSelector = $( 'level' );
		levelSelector.selectedIndex = this.level;
		
		var orientation = $( 'orientLock' );
		orientation.checked = this.orientationLock;
		
		var accelerometer = $( 'accelSupport' );
		accelerometer.checked = this.accelerometerSupport;
	},
	
	_applyOptions: function( changeLevel ) {
		if( changeLevel ){
			this._changeLevel();			
		}
		this._lockOrientation();
		this._supportAccelerometer();
	},
	
	_changeLevel: function() {
		puzzle.changeLevel( this.level );
	},
	
	_lockOrientation: function() {
		if( this.orientationLock ) {
			conduitApi.platform.orientation
				.lockOrientation(empty, empty,
						this.degree );
		}
		else {
			conduitApi.platform.orientation
				.unlockOrientation(empty, empty);
		}
	},
	
	_supportAccelerometer: function() {
		// TODO: add support for accelerometer
	},
	
	_readOptions: function(cbDoneRead) 
	{
		function readOptionsFromArray (optionsArr, cbDone) 
		{
			if (optionsArr.length === 0) 
			{
				cbDoneRead([]);
				return;
			}
			
			var option = optionsArr.shift();
			_options._readOption(
			function (value) 
			{
				readOptionsFromArray(optionsArr, function (retValArr) 
				{
					cbDoneRead(retValArr.unshift(value));
				});
			},
			function () 
			{
				readOptionsFromArray(optionsArr, function (retValArr) 
				{
					cbDoneRead(retValArr.unshift(option.defVal));
				});
			}, option.key );
		}
		
		var optionsArr = 
		[
			{
				key: 'puzzle',
				defVal: 1
			},
			{
				key: 'orientation',
				defVal: false
			},
			{
				key: 'degree',
				defVal: 1
			},
			{
				key: 'accelerometer',
				defVal: false
			},
		];
		
		readOptionsFromArray(optionsArr, function (resultArr) 
		{
			_options.level = parseInt(resultArr[0]);
			_options.orientationLock = resultArr[1] === 'true';
			_options.degree = parseInt(resultArr[2]);
			_options.accelerometerSupport = resultArr[3] === 'true';
		});
	},
	
	_readOption: function(cbSuccess, cbFail, key ) {
	    return conduitApi.platform.storage
	        .getItem(cbSuccess, cbFail, key);
	},
	
	_writeOptions: function() {
		if( this._isDirty ){	
			this._writeOption( 'puzzle', this.level );
			this._writeOption('orientation', this.orientationLock);
						
			this._isDirty = false;
		}
	},
	
	_writeOption: function( key, value ) {
	    var saved = conduitApi.platform.storage
	        .setItem(empty, empty, key, String(value));
	    
	    conduitApi.log.info(empty, empty, key + ' saved: ' + saved );
	}
};

var _game = {
	// Initialize game image and buttons
	init: function() {
		// Get a reference to the image
		var div = $('puzzle');
		if (div) {
			// Add the image to the puzzle collection
			
			var options = {
				level: _options.level,
				width: _image.defaultWidth,
				height: _image.defaultHeight
			}
			
			puzzle.init(div, _image.defaultUri, options);

			// Add events
			var scrambleButton = $('scrambleButton');
			scrambleButton.onclick = function() {
				puzzle.scramble();
			}

			var solveButton = $('solveButton');
			solveButton.onclick = function() {
				puzzle.solve();
			}
		}
	},
	
	newGame: function() {
		_app.switchDisplay('mainMenu', 'game');

		conduitApi.app.hideNavigationBar(empty, empty);
		conduitApi.app.showInfoBar(empty, empty);

		conduitApi.platform.gestures
				.registerShake( empty, empty,
					function() {
						puzzle.scramble();
					} );
		
		puzzle.changeLevel( _options.level );
	},
	
	hideGame: function( toId ) {
		conduitApi.app.hideInfoBar(empty, empty);
		_app.switchDisplay('game', toId);
		conduitApi.app.showNavigationBar(empty, empty);

		conduitApi.platform.gestures.unregisterShake(empty, empty);
	}
};

/*
 * Helper Functions
 */
// Shortcut for getting an element by ID
function $(id) {
	return document.getElementById(id);
}

// No-op function for API calls
function empty() {}

// Function to call when document has loaded
window.onload = initApp;